# -*- coding: utf-8 -*-
"""
Module implementing MainWindow.
"""
import os

from PyQt4.QtGui import QMainWindow,  QTreeWidgetItem,  QPixmap, QDesktopServices
from PyQt4.QtCore import Qt, pyqtSignature

import mingus.core.chords as chords
import mingus.core.intervals as intervals
import mingus.core.notes as notes
import mingus.core.diatonic as diatonic
import mingus.core.scales as scales
import mingus.containers.Bar as Bar

from .Ui_mainwindow import Ui_MainWindow
from .about import AboutDialog
from .settings import SettingsDialog
from calc import Calc
from fluid import Fluid
from lily import Lily

scales_list = ["ionian", "dorian", "phrygian",
                    "lydian", "mixolydian", "aeolian", "locrian",
                    "natural_minor", "harmonic_minor", "melodic_minor", 
                    "chromatic", "whole_note", "diminished"]

intervals_list = ["0", "1", "2", "3", "4", "5", "6", 
                        "7", "8", "9", "10", "11"]
                        


class MainWindow(QMainWindow, Ui_MainWindow):
    def __init__(self, parent = None):
        self.keysDisplay = "C"
        self.showSheet = 0
        self.velocity = 127
    
        self.bar = Bar()
        self.calc = Calc()
        self.lily = Lily()
        self.fluid = Fluid()
        self.fluid.init()
        
        QMainWindow.__init__(self, parent)
        self.setupUi(self)

        self.loadKeys()
        self.loadIntervals()
        self.loadScales()
        self.loadChords()

        self.lily.finished.connect(self.loadImage)

    def setup(self, settings):
        self.settings = settings
        self.fluid.setDriver(self.settings.value('Midi/driver'))
        self.fluid.setFont(self.settings.value('Midi/soundfont'))
        #returns path string without the double slash at the end
        self.settingsDir= str(QDesktopServices.storageLocation(QDesktopServices.CacheLocation))[:-1]
        self.imagesDir = self.settingsDir + "qmingus/"
        if not os.path.exists(self.imagesDir):
            os.makedirs(self.imagesDir)
    
    def loadKeys(self):
        keys_index = self.treeWidgetKeys.indexFromItem(self.treeWidgetKeys.currentItem())
        
        for i in range(0, 12):
            key_item = QTreeWidgetItem()
            key_item.setText(0, diatonic.int_to_note(i, self.keysDisplay))
            self.treeWidgetKeys.takeTopLevelItem(i)
            self.treeWidgetKeys.insertTopLevelItem(i,  key_item)

            if (keys_index.row() == -1):
                key_item = self.treeWidgetKeys.topLevelItem(0)
                self.treeWidgetKeys.setCurrentItem(key_item)

            if (i == keys_index.row()): 
                self.treeWidgetKeys.setCurrentItem(key_item)

    def determineInterval(self,  interval):
        if int(interval) < 12:
                result_note = diatonic.int_to_note((self.calc.key_offset + int(interval))%12, str(self.treeWidgetKeys.currentItem().text(0)))
                self.interval_name = intervals.determine(str(self.treeWidgetKeys.currentItem().text(0)), result_note)
        elif int(interval) == 12:
            self.interval_name = "octave"
            
        return self.interval_name
    def loadIntervals(self):
        #intervals_index = self.treeWidgetIntervals.indexFromItem(self.treeWidgetIntervals.currentItem())
        
        for i in range(len(intervals_list)):
            interval_item = QTreeWidgetItem()
            interval_item.setText(0, intervals_list[i])
            result_interval = self.determineInterval(intervals_list[i])
            interval_item.setText(1, result_interval)
            self.treeWidgetIntervals.takeTopLevelItem(i)
            self.treeWidgetIntervals.insertTopLevelItem(i,  interval_item)     
          
        self.treeWidgetIntervals.resizeColumnToContents(0)
        #print intervals_index.row()
            
    def loadScales(self):
        #scales_index = self.treeWidgetScales.indexFromItem(self.treeWidgetScales.currentItem())

        for i in range(len(scales_list)):
            scale_item = QTreeWidgetItem()
            scale_item.setText(0, scales_list[i])
            self.treeWidgetScales.takeTopLevelItem(i)
            self.treeWidgetScales.insertTopLevelItem(i,  scale_item)       

    def loadChords(self):
        #chords_index = self.treeWidgetChords.indexFromItem(self.treeWidgetChords.currentItem())
        i = 0
        while i < len(list(chords.chord_shorthand_meaning.items())):
            for alias, name in sorted(chords.chord_shorthand_meaning.items()):
                chord_item = QTreeWidgetItem()
                chord_item.setText(0, alias)
                self.treeWidgetChords.takeTopLevelItem(i)
                self.treeWidgetChords.insertTopLevelItem(i,  chord_item)
                i = i + 1
          
        self.treeWidgetChords.takeTopLevelItem(0)
        #if (chords_index.row() == -1):
                #chord_item = self.treeWidgetChords.topLevelItem(0)
                #self.treeWidgetChords.setCurrentItem(chord_item)
                
    def loadImage(self):
        if not (self.treeWidgetIntervals.selectedItems()):
            return

        imagePath = self.imagesDir + "bar_im_"
        for i in range(0, len(self.calc.notesList)):
            imagePath += str(self.calc.charList[i])
            
        self.barImage.clear()

        if not os.path.exists(imagePath + ".png"):
            self.lily.setBar(self.bar)
            self.lily.setPath(imagePath)
            self.lily.start()
        else:
            pix = QPixmap(imagePath + ".png")
            pixs = pix.scaled(1020, 1220)
            pixs.scroll(-100, 550,pixs.rect())
            self.barImage.setPixmap(pixs)

    def fillBar(self, notes_list,  char_list):
        self.bar = Bar()
        for i in range(0, len(notes_list)):
            self.bar.place_notes(notes_list[i], len(char_list))
            
        if self.showSheet == 1:
            self.loadImage()
            
    def playBar(self):
        self.fluid.setBar(self.bar)
        self.fluid.start()

    def selectionChanged(self,  caller):
        self.calc.velocity = self.velocity
        key_selected = str(self.treeWidgetKeys.currentItem().text(0))
        intervals_selected = self.treeWidgetIntervals.selectedItems()
        #print intervals_selected
        #for i in range(0, len(intervals_selected)):
            #print intervals_selected[i].text(0)
        #if caller == "key" or "velocity":
            #notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 0)
        

        
        if caller == "chord":
            self.treeWidgetIntervals.clearSelection()
            self.treeWidgetScales.clearSelection()
            if self.treeWidgetChords.currentItem() is None:
                return
            self.treeWidgetChords.setCurrentItem(self.treeWidgetChords.currentItem())
            chord_name = self.treeWidgetKeys.currentItem().text(0) + self.treeWidgetChords.currentItem().text(0)
            chord_list = chords.from_shorthand(str(chord_name))
            root_note =  notes.note_to_int(chord_list[0])

            for i in range(0,  len(chord_list)):
                chord_note = notes.note_to_int(chord_list[i]) - root_note
                if chord_note < 0:
                    chord_note += 12
                found_items = self.treeWidgetIntervals.findItems(str(chord_note),  Qt.MatchExactly)
                for x in range(0, len(found_items)):
                    self.treeWidgetIntervals.setCurrentItem(found_items[x])
                    
            intervals_selected = self.treeWidgetIntervals.selectedItems()
                
        elif caller == "scale":
            self.treeWidgetIntervals.clearSelection()
            self.treeWidgetChords.clearSelection()
            scale_name = self.treeWidgetScales.currentItem().text(0)
            scale_list = getattr(scales, str(scale_name))(key_selected)
            root_note =  notes.note_to_int(scale_list[0])
            for i in range(0,  len(scale_list)):
                scale_note = notes.note_to_int(scale_list[i]) - root_note
                if scale_note < 0:
                    scale_note += 12
                found_items = self.treeWidgetIntervals.findItems(str(scale_note),  Qt.MatchExactly)
                for x in range(0, len(found_items)):
                    self.treeWidgetIntervals.setCurrentItem(found_items[x])
                    
            intervals_selected = self.treeWidgetIntervals.selectedItems()
            #notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 0)
        if self.pushButtonI1.isChecked():
            notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 1)
        elif self.pushButtonI2.isChecked():
            notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 2)
        elif self.pushButtonI3.isChecked():
            notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 3)
        else:
            notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 0)
        
        if caller == "interval":
            self.treeWidgetChords.clearSelection()
            self.treeWidgetScales.clearSelection()
            #notes_list, char_list,  diff_list =  self.calc.intervals(key_selected,  intervals_selected, 0)
            if  len(intervals_selected) >= 2 <=6:
                self.findChord()
            if len(intervals_selected) >= 7:
                self.findScale()
        
        self.formatHTML(char_list, diff_list)
 
        self.fillBar(notes_list, char_list)
        if self.pushButtonLoop.isChecked() == True:
            self.playBar()
            
    def formatHTML(self, char_list, diff_list):       
        char_string = ''
        for i in range(0, len(char_list)):
            char_string += char_list[i] + " "
        notes_string = "<p align=\"center\"><sitempan style=\""
        notes_string_size = 16
        if len(char_string) > 32:
            notes_string_size = 11
        notes_string += "color:#00A000;font-size:"
        notes_string += str(notes_string_size)
        notes_string += "pt;\"><b>"
        notes_string += char_string
        notes_string += "</b></p>"
        self.textEditNotes.setHtml(notes_string)
        
        diff_string = "<p align=\"center\"><sitempan style=\""
        diff_string += "color:#00A0B8;font-size:11pt;\"><b>"
        for i in range(1, len(diff_list)):
            diff_string += str(diff_list[i]) + " "
        diff_string += "</b></p>"
        self.textEditNumbers.setHtml(diff_string)
        

    def findChord(self):
        chord_list = self.calc.chords()
        for i in range(0, len(chord_list)):
            chord_name = chord_list[i]
            key_len = len(self.treeWidgetKeys.currentItem().text(0))
            chord_type = chord_name[key_len:]
            found_items = self.treeWidgetChords.findItems(chord_type,  Qt.MatchExactly)

            for x in range(0, len(found_items)):
                self.treeWidgetChords.setCurrentItem(found_items[x])

    def findScale(self):
        scale_list = self.calc.scales()
        for i in range(0, len(scale_list)):
            scale = scale_list[i]
            key_len = len(self.treeWidgetKeys.currentItem().text(0)) + 1
            scale_name = scale[key_len:]
            found_items =  self.treeWidgetScales.findItems(scale_name,  Qt.MatchExactly)
        
            for x in range(0, len(found_items)):
                self.treeWidgetScales.setCurrentItem(found_items[x])


    @pyqtSignature("")
    def on_pushButtonPlay_pressed(self):
        self.playBar()

    @pyqtSignature("bool")
    def on_pushButtonLoop_toggled(self, checked):
       self.fluid.toggleLoop(checked)

    @pyqtSignature("")
    def on_pushButtonFlat_clicked(self):
        self.keysDisplay = "Db"
        self.loadKeys()
        self.pushButtonFlat.setChecked(1)
        self.pushButtonSharp.setChecked(0)
        #self.pushButtonI1.setChecked(0)
        #self.pushButtonI2.setChecked(0)
        #self.pushButtonI3.setChecked(0)
        self.selectionChanged("key") 
    
    @pyqtSignature("")
    def on_pushButtonSharp_clicked(self):
        self.keysDisplay = "C"
        self.loadKeys()
        self.pushButtonSharp.setChecked(1)
        self.pushButtonFlat.setChecked(0)
        #self.pushButtonI1.setChecked(0)
        #self.pushButtonI2.setChecked(0)
        #self.pushButtonI3.setChecked(0)
        self.selectionChanged("key") 
       
    @pyqtSignature("")
    def on_pushButtonI1_clicked(self):
        #self.pushButtonI1.setChecked(1)
        self.pushButtonI2.setChecked(0)
        self.pushButtonI3.setChecked(0)
        self.selectionChanged("chord") 
    
    @pyqtSignature("")
    def on_pushButtonI2_clicked(self):
        self.pushButtonI1.setChecked(0)
        #self.pushButtonI2.setChecked(1)
        self.pushButtonI3.setChecked(0)
        self.selectionChanged("chord")  

    @pyqtSignature("")
    def on_pushButtonI3_clicked(self):
        self.pushButtonI1.setChecked(0)
        self.pushButtonI2.setChecked(0)
        #self.pushButtonI3.setChecked(1)
        self.selectionChanged("chord")   

    @pyqtSignature("")
    def on_pushButtonInfo_clicked(self):
        about = AboutDialog()
        about.exec_()

    @pyqtSignature("")
    def on_pushButtonSettings_clicked(self):
        settings = SettingsDialog()
        settings.loadSettings(self.settings)
        settings.exec_()
        
    @pyqtSignature("int")
    def on_spinBoxBPM_valueChanged(self, p0):
        self.fluid.setBpm(p0)
        
    @pyqtSignature("int")
    def on_verticalSliderVelocity_valueChanged(self, value):
        self.velocity = value
        self.selectionChanged("velocity")
    
    @pyqtSignature("bool")
    def on_pushButtonSheet_clicked(self, checked):
        if checked == 0:
            self.showSheet = 0
            self.setMinimumSize(590,  340)
            self.setMaximumSize(590,  340)
            self.resize(590,  340)
            
            print self.size()
            self.barImage.clear()
        else:
            self.showSheet = 1
            self.setMinimumSize(590,  430)
            self.setMaximumSize(590,  430)
            self.resize(590,  430)
            self.loadImage()

    @pyqtSignature("QTreeWidgetItem*, int")
    def on_treeWidgetKeys_itemPressed(self, item, column):
        self.selectionChanged("key")   
        #self.pushButtonI1.setChecked(0)
        #self.pushButtonI2.setChecked(0)
        #self.pushButtonI3.setChecked(0)
        if self.treeWidgetIntervals.topLevelItemCount() != 0:
            for i in range(len(intervals_list)):
                interval_item = self.treeWidgetIntervals.topLevelItem(int(intervals_list[i]))
                result_interval = self.determineInterval(intervals_list[i])
                interval_item.setText(1, result_interval)

    @pyqtSignature("QTreeWidgetItem*, int")
    def on_treeWidgetChords_itemPressed(self, item, column):
        self.selectionChanged("chord")   
    
    @pyqtSignature("QTreeWidgetItem*, int")
    def on_treeWidgetIntervals_itemPressed(self, item, column):
        #if self.interval_name != "octave":
        self.selectionChanged("interval")   
        
    @pyqtSignature("QTreeWidgetItem*, int")
    def on_treeWidgetScales_itemPressed(self, item, column):
        self.selectionChanged("scale")   
